define([
    "dojo/_base/declare",
    "thornberry/app/Component/_EditorComponent",
    "thornberry/app/Component/_TableFieldEditorMixin",
    "thornberry/app/Component/_DataFormMixin",
    "dijit/_TemplatedMixin",
    "dijit/_WidgetsInTemplateMixin",
    "dojo/_base/lang",
    "dojo/_base/array",
    "dojo/text!./Templates/EraEditor.html",
    "./EraItemEditor",
    "./EraRecordView",
    "thornberry/app/utils",
    "dijit/form/DateTextBox",
    "dijit/form/CurrencyTextBox"
],function(
    declare,
    _EditorComponent,
    _TableFieldEditorMixin,
    _DataFormMixin,
    _TemplatedMixin,
    _WidgetsInTemplateMixin,
    lang,
    arrayUtils,
    template,
    EraItemEditor,
    EraRecordView,
    utils
) {
    return declare([_EditorComponent,_TemplatedMixin,_WidgetsInTemplateMixin,_TableFieldEditorMixin,_DataFormMixin], {
        templateString: template,

        itemEditors: null, // key = code
        recordViews: null,

        constructor: function() {
            var t = this;
            t.itemEditors = [];
            t.recordViews = [];
        },

        getInitialProps: function() {
            return lang.mixin(this.inherited(arguments),{
                value: null
            });
        },

        buildRendering: function() {
            var t = this;
            t.inherited(arguments);

            t.addTableFilter("actionId",function(name,item,value) {
                return (item.active == "Yes" && item.paymentAction == true) || item.id == value;
            });
            t.addTableFilter("paymentTypeId",function(name,item,value) {
                return (item.active == "Yes" && item.IsPrePayment == false) || item.id == value;
            });

            t.addLogic("actionId",function(name,value) {
                var item = t.getSelectedItem("actionId");
                if (item && item.allowPost != 1) {
                    t.setFormValue("paymentTypeId","");
                }
            });

            t.addLogic("adjustmentWriteoff",function(name,value) {
                if (!value.length) {
                    t.setFormValue("adjustmentWriteoffThreshold","");
                    t.setFormValue("adjustmentWriteoffReason","");
                }
            });

            t.addRequirement(["transactionDate","actionId","paymentTypeId","adjustmentWriteoffThreshold","adjustmentWriteoffReason"],true);
            
            t.addDisabled(["adjustmentWriteoffThreshold","adjustmentWriteoffReason"],function(name,values) {
                return !values.adjustmentWriteoff.length;
            });
            t.addDisabled("paymentTypeId",function() {
                var item = t.getSelectedItem("actionId");
                return ((t.props.value && t.props.value.paymentSummary.totalClaims == 0) || (item && item.allowPost != 1));
            });
            t.addDisabled("actionId",function() {
                return (t.props.value && t.props.value.paymentSummary.totalClaims == 0);
            });
        },

        postCreate: function() {
            var t = this;
            t.inherited(arguments);

        },

        refreshRendering: function() {
            var t = this;
            arrayUtils.forEach(t.itemEditors,function(editor) { editor.destroy(); });
            arrayUtils.forEach(t.recordViews,function(view) { view.destroy(); });

            t.itemEditors = [];
            t.recordViews = [];
            
            if (t.props.value) {
                t.props.disabled = t.props.value.isProcessed;
                for (var category in t.props.value.adjustmentSummary) {
                    var firstInSection = true;
                    arrayUtils.forEach(t.props.value.adjustmentSummary[category],function(adjustment) {
                        var editor = new EraItemEditor({
                            props: {
                                value: adjustment,
                                category: category,
                                firstInSection: firstInSection,
                                disabled: t.props.disabled
                            },
                            onChange: function() {
                                t.invalidate("modified");
                            }
                        });
                        t.itemEditors.push(editor);
                        t.adjustmentSummaryContainer.appendChild(editor.domNode);
                        firstInSection = false;
                    });
                }

                arrayUtils.forEach(t.props.value.records,function(record) {
                    var recordView = new EraRecordView({
                        props: {
                            value: record
                        }
                    });
                    t.recordViews.push(recordView);
                    t.recordViewContainer.appendChild(recordView.domNode);
                });

                t.defer("updateBalances");
				
				var constraintDates=[];
				constraintDates.push((t.props.value.minTransactionDate ? utils.conversions.date(null,t.props.value.minTransactionDate,"set") : null))
				constraintDates.push((t.props.value.agingDate ? new Date(new Date(t.props.value.agingDate).setDate(1)) : null))
				var maxDate=new Date(Math.max.apply(null,constraintDates));
                //t.getField("transactionDate").constraints.min = (t.props.value.minTransactionDate ? utils.conversions.date(null,t.props.value.minTransactionDate,"set") : null);
				t.getField("transactionDate").constraints.min = maxDate
            }

            t.inherited(arguments);
        },

        onChange: function() {
            this.updateBalances();
        },

        updateBalances: function() {
            var t = this;
            var value = t.get("value");
            arrayUtils.forEach(t.recordViews,function(view) {
                view.set("props",{
                    paymentAction: value.paymentSummary,
                    adjustmentActions: value.adjustmentSummary
                });
            });
        },

        _setValueAttr: function(value) {
            var t = this;
            if (value != null) {
                value.actionId = value.paymentSummary.actionId;
                value.paymentTypeId = value.paymentSummary.paymentTypeId;
            }
            t.inherited(arguments,[value]);
        },

        _getValueAttr: function(value) {
            var t = this;
            var value = t.inherited(arguments);
            var actionItem = t.getSelectedItem("actionId");
            value.paymentSummary = {
                actionId : value.actionId,
                paymentTypeId : value.paymentTypeId,
                postToAr: (actionItem && actionItem.allowPost ? true : false)
            }
            delete value.actionId;
            delete value.paymentTypeId;
            value.adjustmentSummary = {};
            arrayUtils.forEach(t.itemEditors,function(editor) {
                var editorValue = editor.get("value");
                if (editorValue && (editorValue.actionId || editorValue.paymentTypeId)) {
                    actionItem = editor.getSelectedItem("actionId");
                    editorValue.postToAr = (actionItem && actionItem.allowPost ? true : false);
                    value.adjustmentSummary[editor.get("code")] = editorValue;
                }
            });
            return value;
        }
    });
});